#!/usr/bin/perl

###############################################################################
# This is the SA User Admin program, a web-based program to allow users to    #
# control thier individual SpamAssassin settings, and turn on SA checking     #
# without the intervention of the system administrator.                       #
# 									      #
# Version 0.1                                                                 #
###############################################################################
# Copyright (C) 2002 David L. Weiner <dave@gallowglass.org>                   #
#                                                                             #
# This program is free software; you can redistribute it and/or               #
# modify it under the terms of the GNU General Public License		      #
# as published by the Free Software Foundation; either version 2	      #
# of the License, or (at your option) any later version.		      #
# 									      #
# This program is distributed in the hope that it will be useful,	      #
# but WITHOUT ANY WARRANTY; without even the implied warranty of	      #
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the		      #
# GNU General Public License for more details.				      #
# 									      #
# You should have received a copy of the GNU General Public License	      #
# along with this program; if not, write to the Free Software		      #
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA. #
###############################################################################
# Prequisites:                                                                #
# 									      #
# SpamAssassin (doh!)                                                         #
# CGI.pm                                                                      #
# Carp.pm                                                                     #
# DBI.pm and the appropriate DBD drivers for your SQL backend                 #
# Authen::PAM module (available from CPAN)                                    #
# passwd PAM service                                                          #
# vpopmail.pm (if you're running vpopmail, also available from CPAN)          #
# safecat (if you're using Maildir style mail folders)                        #
# 									      #
# Note: This was written for my setup, which is qmail 1.03 + vpopmail 5.1.4   #
# running on Mandrake Linux 7.2.  I've tried to make this as modular as       #
# possible so only constants and other config variables have to be changed.   #
# If you are running the stable 1.5 release, be aware that the version of     #
# of spamc that is included does not honor the -u flag.  The latest 2.0       #
# development release does, and is required to use spamc for vpopmail-style   #
# domains.  If you are using vpopmail, you will have to apply the seekable    #
# patch, or all your email will start to bounce with lseek errors.  If your   #
# going to use procmail AND Maildir style folders, consider using safecat in  #
# your procmail ruleset.  I have heard that procmail's Maildir support works  #
# but doesn't follow the Maildir specification.                               #
###############################################################################
# Features:                                                                   #
# 									      #
# * No need to maintain seperate username and passwords.  Authenticates       #
#   system users via PAM, using the underlying system authentication          #
#   mechanism, and vpopmail-style users from the underlying vpopmail user     #
#   database.                                                                 #
# 									      #
# * The ability of individual users to turn SA checking on or off without     #
#   system administrator intervention.                                        #
# 									      #
# * Reads the SA global configuration files to get the SQL DSN, username      #
#   and password.                                                             #
# 									      #
# * Ignores the global whitelist_from and blacklist_from when displaying the  #
#   users lists.                                                              #
# 									      #
# * Uses <textarea></textarea> for whitelist_from and blacklist_from to       #
#   facilitate adding or deleting more than one addresses at the same time.   #
###############################################################################


# write good code

use strict;
use vpopmail;
use CGI qw(:standard escapeHTML :html3);
use CGI::Carp qw(fatalsToBrowser);
use DBI;
use Authen::PAM;
use Mail::SpamAssassin;

############################################
# Change these to match your configuration #
############################################

# This is what to put at the begining of the line in your 
# .qmail, .forward or equivelient

use constant SPAMC => '|/var/qmail/bin/preline /usr/bin/spamc -f ';

# This is what to place at the end of the line

use constant SPAMC2 => '| /usr/bin/safecat $HOME/Maildir/tmp $HOME/Maildir/new';

# If you're using vpopmail, this is the default entry for a domains .qmail-default

use constant VPOP => "| /var/qmail/vpopmail/bin/vdelivermail \'\' bounce-no-mailbox";

# The name of the file to place the call to spamc/spamassassin in

use constant USERFILE => '.qmail';
use constant VPOPFILE => '.qmail-default';

# Used for branding

our $IMGSRC = '/images/logo.gif';
our $IMGALT = 'WebMasters, Inc.';
our $APPTITLE = 'Spam Tagging';
our $STYLESRC = '/sa/spamassassin.css';


# make the script safer

delete @ENV{qw{IFS CDPATH ENV BASH_ENV}};
$ENV{PATH}="/bin:/usr/bin:/usr/local/bin";

# Some globals we'll need later

our $username;
our $password;
our $pamh; 
our $loginerror;
our $sa_status;
our $sa_username;
our $sa_cookie;
our $required_hits;
our $rewrite_subject; 
our $report_header;
our $use_terse_report;
our $defang_mime;
our $whitelist_from;
our $blacklist_from;
our $enable_sa;
our $disable_sa;

# Change this if you change the name of this script

our $cgi_self = "index.cgi";

# Create the new SA object

our $sa = Mail::SpamAssassin->new({dont_copy_prefs => 1,local_tests_only => 1});

# Read in the global configuration

$sa->compile_now();

sub pam_conv_func {

# Used by the call to PAM to auth a user

	my @res;
	while ( @_ ) {
		my $code = shift;
		my $msg = shift;
		my $ans = "";


		$ans = $username if ($code == PAM_PROMPT_ECHO_ON() );
		$ans = $password if ($code == PAM_PROMPT_ECHO_OFF() );


		push @res, (PAM_SUCCESS(),$ans);
	} # while ( @_ )
	push @res, PAM_SUCCESS();
	return @res;

} # sub pam_conv_func

sub do_auth {

# Authenticate a user

	($username, $password) = @_;
	
	if ($username =~ /\@/) {
	
		if ($username !~ /^postmaster/i) {
		
		# Only the postmaster account is allowed to log in.
		
			$loginerror = 2;
			
			return;
			
		}
	
		# This is a vpopmail user, split it into username and domain
		
		my ($vusername, $vdomain) = split /\@/, $username;
		
                if ( vauth_user($vusername, $vdomain, $password, undef) ) {

		# User is authenticated
		
                       $sa_status="0";
                       
		} else {
		
			$loginerror = 1;
			
                } # if ( vauth_user($vusername, $vdomain, $password, undef) )		
		
	} else {
	
		# Not a vpopmail user, so it must be a system user.  Use PAM to auth the user
		
		ref($pamh = new Authen::PAM("passwd", $username, \&pam_conv_func)) ||
			die "Error code $pamh during PAM init!";
			
		if ($pamh->pam_authenticate == 0) {
		
		# User is authenticated
			
			$sa_status="0";
		
		} else {
		
			$loginerror = 1;
			
		} # if ($pamh->pam_authenticate == 0)
		
	} # if ($username =~ /\@/)
	
} # sub do_auth

sub update_db {

	my $cmd;

	# Get the DBI settings
	
	my $user_scores_dsn = $sa->{conf}->{user_scores_dsn};
	my $user_scores_sql_username = $sa->{conf}->{user_scores_sql_username};
	my $user_scores_sql_password = $sa->{conf}->{user_scores_sql_password};
	
	my $dbh = DBI->connect($user_scores_dsn, $user_scores_sql_username, $user_scores_sql_password)
		or die "Can not connect: " . $DBI::errstr;
	
	# update users's rewrite_subject 
	if(!$rewrite_subject) { $rewrite_subject = 0; }
	$cmd = "delete from userpref where username = '$username' " .
		"AND preference = 'rewrite_subject'";
	$dbh->do($cmd);
	$cmd = "insert into userpref (username, preference, value) " .
		"VALUES ('$username', 'rewrite_subject', '$rewrite_subject')";
	$dbh->do($cmd);

	# update users's report_header 
	if(!$report_header) { $report_header = 0; }
	$cmd = "delete from userpref where username = '$username' " .
		"AND preference = 'report_header'";
	$dbh->do($cmd);
	$cmd = "insert into userpref (username, preference, value) " .
		"VALUES ('$username', 'report_header', '$report_header')";
	$dbh->do($cmd);

	# update users's defang_mime 
	if(!$defang_mime) { $defang_mime = 0; }
	$cmd = "delete from userpref where username = '$username' " .
		"AND preference = 'defang_mime'";
	$dbh->do($cmd);
	$cmd = "insert into userpref (username, preference, value) " .
		"VALUES ('$username', 'defang_mime', '$defang_mime')";
	$dbh->do($cmd);

	# update user's use_terse_report
	if(!$use_terse_report) { $use_terse_report = 0; }
	$cmd = "delete from userpref where username = '$username' " .
		"AND preference = 'use_terse_report'";
	$dbh->do($cmd);
	$cmd = "insert into userpref (username, preference, value) " .
		"VALUES ('$username', 'use_terse_report', '$use_terse_report')";
	$dbh->do($cmd);

	# update users's required_hits 
	if(!$required_hits) { $required_hits = 5; }
        $cmd = "delete from userpref where username = '$username' " .
                "AND preference = 'required_hits'";
        $dbh->do($cmd);
	$cmd = "insert into userpref (username, preference, value) " .
		"VALUES ('$username', 'required_hits', '$required_hits')";
	$dbh->do($cmd);	
	
	# update user's whitelist_from
	
	chomp($whitelist_from); # never assume :)
	
	$cmd = "delete from userpref where username = '$username' " .
		"AND preference = 'whitelist_from'";
	$dbh->do($cmd);
	
	foreach my $addr (split /\s/, $whitelist_from) {
		
		if ($addr =~ /^[a-zA-Z0-9_\.\-]+@[a-zA-Z0-9\-]+\.[a-zA-Z0-9\-\.]+$/ ||
		    $addr =~ /^\*@[a-zA-Z0-9\-]+\.[a-zA-Z0-9\-\.]+$/) { 
		    
		    	# It's a valid email address
		
			$cmd = "insert into userpref (username, preference, value) " .
				"values ('$username', 'whitelist_from', '$addr')";
			$dbh->do($cmd);
			
		} # if ($addr =~ /^[a-zA-Z0-9_\.\-]+@[a-zA-Z0-9\-]+\.[a-zA-Z0-9\-\.]+$/ ||
		
	} # foreach my $addr (split /\r\n/, $whitelist_from)
	
	# update user's blacklist_from
	
	chomp($blacklist_from); # never assume :)
	
	$cmd = "delete from userpref where username = '$username' " .
		"AND preference = 'blacklist_from'";
	$dbh->do($cmd);
	
	foreach my $addr (split /\r\n/, $blacklist_from) {
		
		if ($addr =~ /^[a-zA-Z0-9_\.\-]+@[a-zA-Z0-9\-]+\.[a-zA-Z0-9\-\.]+$/ ||
		    $addr =~ /^\*@[a-zA-Z0-9\-]+\.[a-zA-Z0-9\-\.]+$/) { 
		    
		    # It's a valid email address
		
			$cmd = "insert into userpref (username, preference, value) " .
				"values ('$username', 'blacklist_from', '$addr')";
			$dbh->do($cmd);
			
		} # if ($addr =~ /^[a-zA-Z0-9_\.\-]+@[a-zA-Z0-9\-]+\.[a-zA-Z0-9\-\.]+$/ ||
		
	} # foreach my $addr (split /\r\n/, $blacklist_from) 
	
	$dbh->disconnect();
	
	# Check for enable/disable
	
	if ($enable_sa) {
	
		if ($username =~ /\@/) {

			# They're a vpopmail user.  First, we need to know the domain directory

				my @lines;

				my ($vuser,$vdom) = split /\@/, $username;

				my $filename = vgetdomaindir($vdom);

				$filename .= VPOPFILE;
				
				# first, suck in everything except the vdeliver line

				open(DOTQMAIL, $filename);

				while (<DOTQMAIL>) {
					
					my $line = $_;
					
					$line = SPAMC . "-u postmaster\@$vdom " . $_ if ($_ =~ /vdelivermail/);

					push(@lines, $line); 

				} # while (<DOTQMAIL>)

				close(DOTQMAIL);

				# Now, open the file for writing and push it back out
				
				my $cleanfile = $1 if ($filename =~ /(.*)/);
				
				open(DOTQMAIL,">$cleanfile");
				
				while (my $line = pop(@lines)) {
				
					print DOTQMAIL $line;
					
				} # while (my $line = pop(@lines))
				
				close(DOTQMAIL);
				
			my ($uidnum, $gidnum) = (getpwnam("vpopmail"))[2,3];
			
			my $uid = $1 if ($uidnum =~ /(.*)/);
			my $gid = $1 if ($gidnum =~ /(.*)/);
			
			chown($uid, $gid, $cleanfile);
				
		} else {

			# They're a system user.  Get the users home dir from
			# /etc/passwd (or equiv) and then look at their .qmail file
			
			my @lines;
			
			my $line = SPAMC . SPAMC2 . "\n";
			
			push(@lines, $line); 

			my @pwent = getpwnam($username);

			my $filename = $pwent[7] . USERFILE;

			open(DOTQMAIL, $filename);

			while (<DOTQMAIL>) {

				push(@lines, $_);

			} # while (<DOTQMAIL>)

			close(DOTQMAIL);
			
			# Now, open the file for writing and push it back out

			my $cleanfile = $1 if ($filename =~ /(.*)/);

			open(DOTQMAIL,">$cleanfile");

			while (my $line = pop(@lines)) {

				print DOTQMAIL $line;

			} # while (my $line = pop(@lines))

			close(DOTQMAIL);
			
			my ($uidnum, $gidnum) = (getpwnam($username))[2,3];
			
			my $uid = $1 if ($uidnum =~ /(.*)/);
			my $gid = $1 if ($gidnum =~ /(.*)/);
			
			chown($uid, $gid, $cleanfile);

		} # if ($username =~ /\@/)
		
	} # if ($enable_sa)		

	if ($disable_sa) {
	
		if ($username =~ /\@/) {

			# They're a vpopmail user.  First, we need to know the domain directory

				my @lines;

				my ($vuser,$vdom) = split /\@/, $username;

				my $filename = vgetdomaindir($vdom);

				$filename .= VPOPFILE;
				
				# first, suck in everything except the vdeliver line
				
				my $line = VPOP . "\n";
				
				push(@lines,$line);

				open(DOTQMAIL, $filename);

				while (<DOTQMAIL>) {
					
					push(@lines, $_) if ($_ !~ /vdelivermail/); 

				} # while (<DOTQMAIL>)

				close(DOTQMAIL);

				# Now, open the file for writing and push it back out
				
				my $cleanfile = $1 if ($filename =~ /(.*)/);
				
				open(DOTQMAIL,">$cleanfile");
				
				while (my $line = pop(@lines)) {
				
					print DOTQMAIL $line;
					
				} # while (my $line = pop(@lines))
				
				close(DOTQMAIL);
				
			my ($uidnum, $gidnum) = (getpwnam("vpopmail"))[2,3];
			
			my $uid = $1 if ($uidnum =~ /(.*)/);
			my $gid = $1 if ($gidnum =~ /(.*)/);
			
			chown($uid, $gid, $cleanfile);
				
		} else {

			# They're a system user.  Get the users home dir from
			# /etc/passwd (or equiv) and then look at their .qmail file
			
			my @lines;
			
			my @pwent = getpwnam($username);

			my $filename = $pwent[7] . USERFILE;

			open(DOTQMAIL, $filename);

			while (<DOTQMAIL>) {

				push(@lines, $_) if ($_ !~ /spamc/);

			} # while (<DOTQMAIL>)

			close(DOTQMAIL);
			
			# Now, open the file for writing and push it back out

			my $cleanfile = $1 if ($filename =~ /(.*)/);

			open(DOTQMAIL,">$cleanfile");

			while (my $line = pop(@lines)) {

				print DOTQMAIL $line;

			} # while (my $line = pop(@lines))

			close(DOTQMAIL);
			
			my ($uidnum, $gidnum) = (getpwnam($username))[2,3];
			
			my $uid = $1 if ($uidnum =~ /(.*)/);
			my $gid = $1 if ($gidnum =~ /(.*)/);
			
			chown($uid, $gid, $cleanfile);

		} # if ($username =~ /\@/)
		
	} # if ($disable_sa)		
	
} # sub update_db

sub show_login {

print <<"EOF";
<form action="$cgi_self" method=post>
<input type=hidden name=login value=1>
<table align="center">
	<tr>
		<td align="center" colspan="2"><img src="$IMGSRC" alt="$IMGALT"><br>
			<font class=mainheading>$APPTITLE Settings</font>
		</td>
	</tr>
	<tr>
		<td colspan=2 align="center"><font class=mainheading>User Login</font></td>
	</tr>
	<tr>
		<td colspan=2 align="center"><font class=main>Please enter your username and password</font></td>
	</tr>
EOF
if ($loginerror == 1) {
print <<"EOF";
	<tr>
		<td colspan=2 align="center"><font class=red><b>Login Failed, Please try again!</b></font></td>
	</tr>
EOF
}
if ($loginerror == 2) {
print <<"EOF";
	<tr>
		<td colspan=2 align="center"><font class=red><b>Only the Postmaster of a virtual domain is allowed to change $APPTITLE settings.</b></font></td>
	</tr>
EOF
}
print <<"EOF";
	<tr>
		<td align="right"><font class=main>Username:</font></td>
		<td><input name=username type=text size=25 maxlength=100></td>
	</tr>
	<tr>
		<td align="right"><font class=main>Password:</font></td>
		<td><input name=password type=password size=25></td>
	</tr>
	<tr>
		<td colspan=2 align="center"><input type=submit name=submit value="Login"></td>
	</tr>
</table>
</form>
EOF

} # sub show_login

sub get_prefs {

	my $rwcheck;
	my $headcheck;
	my $tersecheck;
	my $defangcheck;
	my $sa_active = '<font class="red">Inactive</font><br><font class="main"><input type=checkbox name="enable_sa" value=1> Enable $APPTITLE</font>';

	# Clear any whitelist addresses that may be in the global config
	# before loading the users prefs

	my $temp = $sa->{conf}->{whitelist_from};
	my $count = @$temp;

	for (my $i=0; $i < $count; $i++) { 
		$sa->{conf}->{whitelist_from}[$i]="";
	}

	# And now we do the same with the blacklist

	my $temp = $sa->{conf}->{blacklist_from};
	my $count = @$temp;

	for (my $i=0; $i < $count; $i++) { 
		$sa->{conf}->{blacklist_from}[$i]="";
	}

	# Load the user prefs from the db

	$sa->load_scoreonly_sql($sa_username);
	
	# Put the user preferences in easy to use vars

	$required_hits = $sa->{conf}->{required_hits};
	$rewrite_subject = $sa->{conf}->{rewrite_subject};
	
	if ($rewrite_subject == 1) { $rwcheck=" checked"; }
	
	$report_header = $sa->{conf}->{report_header};
	
	if ($report_header == 1) { $headcheck=" checked"; }
	
	$use_terse_report = $sa->{conf}->{use_terse_report};
	
	if ($use_terse_report == 1) { $tersecheck = " checked"; }
	
	$defang_mime = $sa->{conf}->{defang_mime};
	
	if ($defang_mime == 1) { $defangcheck = " checked"; }
	
	my $whitelist_from = $sa->{conf}->{whitelist_from};
	my $blacklist_from = $sa->{conf}->{blacklist_from};
	
	# Check if they are using SpamAssassin
	
	if ($username =~ /\@/) {
	
		# They're a vpopmail user.  First, we need to know the domain directory
		
		if ($username =~ /^postmaster/i) {
		
			my ($vuser,$vdom) = split /\@/, $username;

			my $filename = vgetdomaindir($vdom);

			$filename .= "/.qmail-default";

			open(DOTQMAIL, $filename);

			while (<DOTQMAIL>) {

				$sa_active= '<font class="green">Active</font><br><font class="main"><input type=checkbox name="disable_sa" value=1> Disable ' . $APPTITLE . '</font>' if ($_ =~ /spamc/); 

			} # while (<DOTQMAIL>)

			close(DOTQMAIL);

			$sa_active .= ' <font class="main"><em>(Note: These are the global settings for your domain)</em></font>';
			
		} else {
		
			$sa_active = '<font class="red">Unknown</font>';
		
		} # if ($username =~ /^postmaster/i)
		
	} else {
	
		# They're a system user.  Get the users home dir from
		# /etc/passwd (or equiv) and then look at their .qmail file
	
		my @pwent = getpwnam($username);
		
		my $filename = $pwent[7] . "/.qmail";
		
		open(DOTQMAIL, $filename);
		
		while (<DOTQMAIL>) {
		
			$sa_active= '<font class="green">Enabled</font><br><font class="main"><input type=checkbox name="disable_sa" value=1> Disable ' . $APPTITLE . '</font>' if ($_ =~ /spamc/); 
			
		} # while (<DOTQMAIL>)
		
		close(DOTQMAIL);
		
	} # if ($username =~ /\@/)
		
		

print <<"EOB";
<form action="$cgi_self" method=post>
<input type=hidden name=update value=1>
<table align="center" width="90%">
	<tr>
		<td align="center" colspan="3"><img src="$IMGSRC" alt="$IMGALT"><br>
			<font class=mainheading>$APPTITLE Settings</font>
		</td>
	</tr>
	<tr>
		<td align=left>
		<font class=main>Logged in as: <b>$sa_username</b></font>
		</td>
		<td>&nbsp;</td>
		<td align=right>
			<a class=main href="$cgi_self?logout=-1">Logout</a>
		</td>
	</tr>
	<tr>
		<td colspan="3" align="left"><font class="main">$APPTITLE Status:</font> <b>$sa_active</b></td>
	</tr>
	<tr>
		<td colspan="3"><hr></td>
	</tr>
	<tr>
		<td colspan="3" align="center"><font class=main>From here you can change the settings that control how the $APPTITLE will mark your e-mail.
		</td>
	</tr>
	<tr>
		<td colspan="3"><hr></td>
	</tr>	
	<tr>
		<td colspan="3" align="center"><font class=main>Below is your list of sender addresses that are either <b>Allowed</b> or <b>Denied</b> passage regardless of the Spam score.</font><br><br>
		</td>
	</tr>
	<tr class=main>
		<th align="center"><font class=main><B>Allowed Addresses</B></font></th>
		<th>&nbsp;</th>
		<th align="center"><font class=main><B>Denied Addresses</B></font></th>
	</tr>
	<tr class=main>
		<td align="center" valign="top"><font class=main><textarea name="whitelist_from" cols="30" rows="10">
EOB

# Run through the whitelist and blacklist, and build the textarea boxes

foreach my $addr (@$whitelist_from) {

	print "$addr\n" if ($addr !~ /^$/);
	
}

print <<"EOB";
</textarea></font></td>
		<td>&nbsp;</td>
		<td align="center" valign="top"><font class=main><textarea name="blacklist_from" cols="30" rows="10">
EOB

foreach my $addr (@$blacklist_from) {

	print "$addr\n" if ($addr !~ /^$/);
	
}

print <<"EOB";
		</textarea></font></td>
	</tr>
EOB

print <<"EOB";
	<tr>
		<td colspan=3 align="center"><br><input name=submit type=submit value="Update Settings"></td>
	</tr>

	<tr>
		<td colspan="3"><hr></td>
	</tr>	
	<tr>
		<td colspan="3" align="center"><font class=mainheading>General Settings</font></td>
	</tr>
	<tr>
		<td colspan="3"><hr></td>
	</tr>
</table>	
EOB
print <<"EOB";
<table align="center" width="90%">
	<tr>
		<td><input type=text size=2 maxlength=2 name=required_hits value="$required_hits"></td>
		<td nowrap><font class=main><B>Required Hits</b></td>
		<td><font class=main>Once all $APPTITLE tests have been run, the resulting score is matched against this value, and if greater than this value, the e-mail message is marked as spam.</td>
	</tr>
	<tr>
		<td colspan=2>&nbsp;</td>
		<td nowrap><font class=main><br>The default value for Required Hits is 5, which is quite aggressive.<br>A value of 10 will produce less false positives.<br>The average SPAM score is around 15 to 16.</td>
	</tr>
	<tr>
		<td><input type=checkbox name="rewrite_subject" value=1$rwcheck></td>
		<td nowrap><font class=main><b>Rewrite Subject Line</b></td>
		<td><font class=main><br>The $APPTITLE will re-write the subject line of your e-mail that has detected to be spam by prefixing the original subject with: *****SPAM*****.  To turn off this feature, un-check this option.
		</td>
	</tr>
	<tr>
		<td><input type=checkbox name="report_header" value=1$headcheck></td>
		<td nowrap><font class=main><B>Report in Header</b></td>
		<td><font class=main><br>$APPTITLE will create a report when it detects an e-mail message to be spam.  This report is normaly prepended to the e-mail message.  To have the report written as part of the e-mail header, select this option.
		</td>
	</tr>
	<tr>
		<td><input type=checkbox name="defang_mime" value=1$defangcheck></td>
		<td nowrap><font class=main><b>De-HTML Spam<b></td>
		<td><font class=main><br>When a e-mail in HTML format is detected as spam, the $APPTITLE will convert the message to text. To keep the e-mail in it's original HTML form, de-select this option.
		</td>
	</tr>
	<tr>
		<td><input type=checkbox name="use_terse_report" value=1$tersecheck></td>
		<td nowrap><font class=main><b>Short Report</b></td>
		<td><font class=main>This options will tell the $APPTITLE to use a shorter spam report once the message has been found to be spam.
		</td>
	</tr>
	<tr>
		<td colspan=3 align="center"><br><input name=submit type=submit value="Update Settings"></td>
	</tr>
</table>
</form>	
EOB
} # sub get_prefs

$sa_status = cookie("sa_status");
$sa_username = cookie("sa_username");
$username = $sa_username;

my $login = param("login");
my $logout = param("logout");
my $update = param("update");
$required_hits = param("required_hits");
$rewrite_subject = param("rewrite_subject");
$report_header = param("report_header");
$use_terse_report = param("use_terse_report");
$defang_mime = param("defang_mime");
$whitelist_from = param("whitelist_from");
$blacklist_from = param("blacklist_from");
$enable_sa = param("enable_sa");
$disable_sa = param("disable_sa");

chomp $whitelist_from;
chomp $blacklist_from;

if ($logout == -1 ) {

	$sa_cookie = cookie(-name => "sa_username", -value => "");
	$sa_status = "-1";
}

if ($login) {

	do_auth(param("username"),param("password"));
	
	if ($sa_status eq "0") {
	
		$sa_cookie = cookie(-name => "sa_username", -value => param("username"));
		$sa_username = param("username");
		
	} # if ($sa_status eq "0")
	
} # if ($login)

if ($sa_cookie) {
	
	print header(-type => "text/html", -cookie => $sa_cookie, -expires => "-1d");
	
} else {

	print header(-type => "text/html", -expires => "-1d");
	
} # if ($cookie) 

print start_html(-title => "$APPTITLE Settings", -bgcolor=>"#FFFFFF", -style=>{src=>$STYLESRC});

if ($update) {

	$sa_status = "0";
	update_db();

} # if ($update)	

if ($sa_status ne "0") {

	show_login();
	
} else {

	get_prefs();

} # if ($sa_status ne "0")

print end_html;

